# from studio.app.common.wrappers import wrapper_dict as common_wrapper_dict
from studio.app.optinist.wrappers import wrapper_dict as optinist_wrapper_dict

wrapper_dict = {}
# wrapper_dict.update(**common_wrapper_dict)
wrapper_dict.update(**optinist_wrapper_dict)
